
%define build_all 1
%define build_arm %{build_all}
%define build_aarch64 %{build_all}

%global multilib_64_archs aarch64

# The gcc versioning information.  In a sed command below, the specfile winds
# pre-release version numbers in BASE-VER back to the last actually-released
# number.
%global YEAR 2013
%global MONTH 05
%global RELEASE %{YEAR}.%{MONTH}
%global gcc_version 4.8

# Note, gcc_release must be integer, if you want to add suffixes to
# %{release}, append them after %{gcc_release} on Release: line.
%global gcc_release %{RELEASE}

Summary: Linaro Cross C compiler for Fedora
Name: fedora-linaro-cross-gcc
Version: %{gcc_version}.%{gcc_release}
Release: 1%{?dist}
License: GPLv3+ and GPLv3+ with exceptions and GPLv2+ with exceptions and LGPLv2+ and BSD
Group: Development/Languages
URL: http://gcc.gnu.org

%define srcdir gcc-linaro-%{gcc_version}-%{gcc_release}
Source0: %{srcdir}.tar.bz2

Patch0: gcc47-hack.patch
Patch1: gcc47-c++-builtin-redecl.patch
Patch2: gcc47-java-nomulti.patch
Patch3: gcc47-ppc32-retaddr.patch
Patch5: gcc47-rh330771.patch
Patch6: gcc47-i386-libgomp.patch
Patch7: gcc47-sparc-config-detection.patch
Patch8: gcc47-libgomp-omp_h-multilib.patch
Patch9: gcc47-libtool-no-rpath.patch
Patch11: gcc47-pr38757.patch
Patch13: gcc47-no-add-needed.patch
Patch14: gcc47-ppl-0.10.patch
Patch15: gcc47-libitm-fno-exceptions.patch

Patch100: cross-intl-filename.patch
# ia64 - http://gcc.gnu.org/bugzilla/show_bug.cgi?id=44553
# m68k - http://gcc.gnu.org/bugzilla/show_bug.cgi?id=53557
# alpha - http://gcc.gnu.org/bugzilla/show_bug.cgi?id=55344
Patch101: cross-gcc-with-libgcc.patch
Patch102: cross-gcc-sh-libgcc.patch

BuildRoot: %{_tmppath}/%{name}-%{version}-%{release}-root-%(%{__id_u} -n)
BuildRequires: binutils >= 2.23.51.0.1
BuildRequires: zlib-devel, gettext, dejagnu, bison, flex, texinfo, sharutils
BuildRequires: cross-binutils-common >= 2.23

# Make sure pthread.h doesn't contain __thread tokens
# Make sure glibc supports stack protector
# Make sure glibc supports DT_GNU_HASH
BuildRequires: glibc-devel >= 2.4.90-13
%ifarch %{multilib_64_archs} sparcv9 ppc
# Ensure glibc{,-devel} is installed for both multilib arches
BuildRequires: /lib/libc.so.6 /usr/lib/libc.so /lib64/libc.so.6 /usr/lib64/libc.so
%endif
BuildRequires: elfutils-devel >= 0.147
BuildRequires: elfutils-libelf-devel >= 0.147
BuildRequires: gmp-devel >= 4.2, mpfr-devel >= 2.3.1, libmpc-devel >= 0.8.1
Provides: bundled(libiberty)

%description
Linaro Cross-build GNU C compiler collection for Fedora.

%package -n fedora-linaro-cross-gcc-common
Summary: Linaro Cross-build GNU C compiler documentation for Fedora
Group: Development/Languages
BuildArch: noarch

%description -n fedora-linaro-cross-gcc-common
Documentation, manual pages and translation files for Linaro cross-build
GNU C compiler targeted for the Fedora environment.

This is the common part of a set of cross-build GNU C compiler packages for
building kernels for other architectures.  No support for cross-building
user space programs is currently supplied as that would massively multiply the
number of packages.

%define do_package() \
%if %2 \
%package -n fedora-linaro-gcc-%1 \
Summary: Linaro Cross-build binary utilities for Fedora on %1 \
Group: Development/Tools \
Requires: fedora-linaro-cross-gcc-common == %{version}-%{release} \
BuildRequires: fedora-linaro-binutils-%1 >= 2.23 \
Requires: fedora-linaro-binutils-%1 >= 2.23 \
%description -n fedora-linaro-gcc-%1 \
Linaro Cross-build GNU C compiler for Fedora. \
%endif

%do_package arm-linux-gnu	%{build_arm}
%do_package aarch64-linux-gnu	%{build_aarch64}

###############################################################################
#
# Preparation
#
###############################################################################
%prep

%setup -q -n %{srcdir} -c
cd %{srcdir}
%patch0 -p0 -b .hack~
%patch1 -p0 -b .c++-builtin-redecl~
%patch2 -p0 -b .java-nomulti~
%patch3 -p0 -b .ppc32-retaddr~
%patch5 -p0 -b .rh330771~
%patch6 -p0 -b .i386-libgomp~
%patch7 -p0 -b .sparc-config-detection~
%patch8 -p0 -b .libgomp-omp_h-multilib~
%patch9 -p0 -b .libtool-no-rpath~
%patch11 -p0 -b .pr38757~
%patch13 -p0 -b .no-add-needed~
%patch15 -p0 -b .libitm-fno-exceptions~
%patch100 -p0 -b .cross-intl~
%patch101 -p1 -b .with-libgcc~
%patch102 -p0 -b .sh-libgcc~

echo 'Linaro %{gcc_version}-%{gcc_release} for Fedora' > gcc/DEV-PHASE

# Default to -gdwarf-4 -fno-debug-types-section rather than -gdwarf-2
sed -i '/UInteger Var(dwarf_version)/s/Init(2)/Init(4)/' gcc/common.opt
sed -i '/flag_debug_types_section/s/Init(1)/Init(0)/' gcc/common.opt
sed -i '/dwarf_record_gcc_switches/s/Init(0)/Init(1)/' gcc/common.opt
sed -i 's/\(may be either 2, 3 or 4; the default version is \)2\./\14./' gcc/doc/invoke.texi

./contrib/gcc_update --touch

LC_ALL=C sed -i -e 's/\xa0/ /' gcc/doc/options.texi

function prep_target () {
    target=$1
    cond=$2

    if [ $cond != 0 ]
    then
	echo $1 >&5
    fi
}

cd ..
(
    prep_target arm-linux-gnu		%{build_arm}
    prep_target aarch64-linux-gnu	%{build_aarch64}
) 5>target.list

n=0
for target in `cat target.list`
do
    n=1
    break
done
if [ $n = 0 ]
then
    echo "No targets selected" >&2
    exit 8
fi

###############################################################################
#
# Build
#
###############################################################################
%build

function config_target () {
    arch=$1
    cross=fedora-linaro-$arch-
    build_dir=$arch

    case $arch in
	arm-*)		target=arm-linux-gnueabi;;
	*)		target=$arch;;
    esac

    echo $arch: target is $target
    #export CFLAGS="$RPM_OPT_FLAGS"

    CONFIG_FLAGS=
    COPT=
    case $arch in
	*)
	    COPT=""
	    ;;
    esac

    mkdir $build_dir
    cd $build_dir

    # We could optimize cross builds size by --enable-shared but the produced
    # binaries may be less convenient in the embedded environment.
    AR_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-ar \
    AS_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-as \
    DLLTOOL_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-dlltool \
    LD_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-ld \
    NM_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-nm \
    OBJDUMP_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-objdump \
    RANLIB_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-ranlib \
    STRIP_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-strip \
    WINDRES_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-windres \
    WINDMC_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-windmc \
    LDFLAGS='-Wl,-z,relro ' \
    ../%{srcdir}/configure \
	--disable-dependency-tracking \
	--disable-silent-rules \
	--prefix=%{_prefix} \
	--exec-prefix=%{_exec_prefix} \
	--bindir=%{_bindir} \
	--sbindir=%{_sbindir} \
	--sysconfdir=%{_sysconfdir} \
	--datadir=%{_datadir} \
	--includedir=%{_includedir} \
	--libdir=%{_libdir}/fedora-linaro \
	--libexecdir=%{_libexecdir}/fedora-linaro \
	--localstatedir=%{_localstatedir} \
	--sharedstatedir=%{_sharedstatedir} \
	--mandir=%{_mandir} \
	--infodir=%{_infodir} \
	--build=%{_target_platform} \
	--host=%{_target_platform} \
	--target=$target \
	--enable-targets=all \
	--program-prefix=$cross \
	--enable-languages=c --with-newlib --without-headers \
	--disable-sjlj-exceptions --with-system-libunwind \
	--disable-nls --disable-threads --disable-shared \
	--disable-libmudflap --disable-libssp --disable-libgomp \
	--disable-libquadmath --disable-gold \
	--disable-decimal-float \
	--enable-checking=$checking \
	--enable-gnu-unique-object \
	--enable-linker-build-id \
	--disable-plugin \
	--enable-nls \
	--with-system-zlib \
	--with-bugurl=http://bugzilla.redhat.com/bugzilla/ \
	--enable-obsolete \
	$CONFIG_FLAGS
    cd ..
}

for target in `cat target.list`
do
    config_target $target
done

function build_target () {
    arch=$1

    AR_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-ar \
    AS_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-as \
    DLLTOOL_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-dlltool \
    LD_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-ld \
    NM_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-nm \
    OBJDUMP_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-objdump \
    RANLIB_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-ranlib \
    STRIP_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-strip \
    WINDRES_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-windres \
    WINDMC_FOR_TARGET=%{_bindir}/fedora-linaro-$arch-windmc \
    make -C $target %{_smp_mflags} tooldir=%{_prefix} all-gcc

    case $arch in
	%{no_libgcc_targets})
	    ;;
	*)
	    make -C $target %{_smp_mflags} tooldir=%{_prefix} all-target-libgcc
	    ;;
    esac

}

for target in `cat target.list`
do
    build_target $target
done

# for documentation purposes only
mkdir stuff
cd stuff

../%{srcdir}/configure \
    --disable-dependency-tracking \
    --disable-silent-rules \
    --prefix=%{_prefix} \
    --exec-prefix=%{_exec_prefix} \
    --bindir=%{_bindir} \
    --sbindir=%{_sbindir} \
    --sysconfdir=%{_sysconfdir} \
    --datadir=%{_datadir} \
    --includedir=%{_includedir} \
    --libdir=%{_libdir}/fedora-linaro \
    --libexecdir=%{_libexecdir}/fedora-linaro \
    --localstatedir=%{_localstatedir} \
    --sharedstatedir=%{_sharedstatedir} \
    --mandir=%{_mandir} \
    --infodir=%{_infodir} \
    --build=%{_target_platform} \
    --host=%{_target_platform} \
    --program-prefix=fedora-linaro-$arch- \
    --enable-languages=c --without-headers \
    --enable-sjlj-exceptions --with-system-libunwind \
    --disable-nls --disable-threads --disable-shared \
    --disable-libmudflap --disable-libssp --disable-libgomp \
    --disable-libquadmath --disable-gold \
    --disable-decimal-float \
    --enable-checking=$checking \
    --enable-gnu-unique-object \
    --enable-linker-build-id \
    --enable-plugin \
    --enable-nls \
    --with-system-zlib \
    --with-bugurl=http://bugzilla.redhat.com/bugzilla/

cd ..

###############################################################################
#
# Installation
#
###############################################################################
%install
rm -rf %{buildroot}

function install_bin () {
    arch=$1

    case $arch in
	%{no_libgcc_targets})	with_libgcc="";;
	*)			with_libgcc="install-target-libgcc";;
    esac

    make -C $arch DESTDIR=%{buildroot} install-gcc ${with_libgcc}
}

for target in `cat target.list`
do
    install_bin $target
done

# For cross-gcc we drop the documentation.
rm -rf %{buildroot}%{_infodir}

# Remove binaries we will not be including, so that they don't end up in
# gcc-debuginfo
FULLPATH=%{buildroot}%{_prefix}/lib/gcc/%{gcc_target_platform}/%{gcc_version}

rm -f %{buildroot}%{_libdir}/{libffi*,libiberty.a}
rm -f %{buildroot}%{_libexecdir}/fedora-linaro/gcc/*/%{gcc_version}/install-tools/{mkheaders,fixincl}
rm -f %{buildroot}%{_prefix}/bin/*-gcc-%{version} || :
rm -f %{buildroot}%{_bindir}/*-ar || :
rm -f %{buildroot}%{_bindir}/*-nm || :
rm -f %{buildroot}%{_bindir}/*-ranlib || :
rmdir  %{buildroot}%{_includedir}

find %{buildroot}%{_datadir} -name gcc.mo |
while read x
do
    y=`dirname $x`
    mv $x $y/fedora-linaro-cross-gcc.mo
done

%find_lang fedora-linaro-cross-gcc

gzip %{buildroot}%{_mandir}/man1/*.1
rm %{buildroot}%{_mandir}/man7/*.7
rmdir %{buildroot}%{_mandir}/man7

# All the installed manual pages and translation files for each program are the
# same, so symlink them to the common package
cd %{buildroot}%{_mandir}/man1
ls
for i in cpp.1.gz gcc.1.gz gcov.1.gz
do
    j=${i#fedora-linaro-}

    for k in *-$j
    do
	if [ $k != $i ]
	then
	    mv $k fedora-linaro-$i
	    ln -s fedora-linaro-$i $k
	fi
    done
done
cd -

function install_lang () {
    arch=$1
    cpu=${arch%%%%-*}

    (
	echo '%%defattr(-,root,root,-)'
	echo '%{_bindir}/'fedora-linaro-$arch'-*'
	echo '%{_bindir}/'$arch'*-*'
	echo '%{_mandir}/man1/'fedora-linaro-$arch'-*'

	case $cpu in
	    *)			target_cpu=$cpu;;
	esac
	echo '%{_libdir}/fedora-linaro/gcc/'$target_cpu'-*/'
	echo '%{_libexecdir}/fedora-linaro/gcc/'$target_cpu'-*/'

    ) >files.$arch
}

for target in `cat target.list`
do
    install_lang $target
done

%define __ar_no_strip $RPM_BUILD_DIR/%{srcdir}/ar-no-strip
cat >%{__ar_no_strip} <<EOF
#!/bin/bash
f=\$2
if [ \${f##*/} = libgcc.a -o \${f##*/} = libgcov.a ]
then
	:
else
	%{__strip} \$*
fi
EOF
chmod +x %{__ar_no_strip}
%undefine __strip
%define __strip %{__ar_no_strip}

###############################################################################
#
# Cleanup
#
###############################################################################
%clean
rm -rf %{buildroot}

###############################################################################
#
# Filesets
#
###############################################################################
%files -n fedora-linaro-cross-gcc-common -f fedora-linaro-cross-gcc.lang
%doc %{srcdir}/COPYING*
%doc %{srcdir}/README
%{_mandir}/man1/fedora-linaro-*

%define do_files() \
%if %2 \
%files -n fedora-linaro-gcc-%1 -f files.%1 \
%endif

%do_files arm-linux-gnu		%{build_arm}
%do_files aarch64-linux-gnu	%{build_aarch64}

%changelog
* Fri Jun 07 2013 Al Stone <ahs3@redhat.com> - 4.8.2013.05-1
- Update to latest Linaro released source
- Refresh patches

* Sun Jan 20 2013 Al Stone <ahs3@redhat.com> - 4.7-linaro-2012120
- Cloned the cross-gcc package for Linaro tools targeting Fedora

* Tue Nov 20 2012 David Howells <dhowells@redhat.com> - 4.7.2-2.aa.20121114
- Fixed the sh64 libgcc compilation.

* Thu Nov 15 2012 David Howells <dhowells@redhat.com> - 4.7.2-1.aa.20121114
- Move to the gcc-4.7-aarch64 SVN branch and add a compiler for the aarch64 arch.
- Disable sjlj exception support.
- Compile libgcc for certain platforms.
- Fix fuzz in sparc-config-detection patch.

* Mon Oct 15 2012 Jon Ciesla <limburgher@gmail.com> - 4.7.1-0.1.20120606.2
- Provides: bundled(libiberty)

* Wed Jul 18 2012 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 4.7.1-0.1.20120606.1
- Rebuilt for https://fedoraproject.org/wiki/Fedora_18_Mass_Rebuild

* Mon Jun 11 2012 David Howells <dhowells@redhat.com> - 4.7.1-0.1.20120606
- Update to gcc-4.7.1-RC-20120606 to fix an ICE in the MIPS compiler.
- The SH64 compilation patches are now redundant.

* Wed May 30 2012 David Howells <dhowells@redhat.com> - 4.7.0-1
- Update to gcc-4.7.0.
- Add patches to fix SH64 compilation.
- Build separate SH and SH64 compilers.

* Thu Mar 22 2012 David Howells <dhowells@redhat.com> - 4.7.0-0.11.4
- Add missing BuildRequires

* Thu Mar 22 2012 David Howells <dhowells@redhat.com> - 4.7.0-0.11.3
- Fix gcc47-ppl-0.10.patch fuzz.

* Thu Mar 22 2012 David Howells <dhowells@redhat.com> - 4.7.0-0.11.2
- Initial import of cross-binutils [BZ 766166].

* Mon Oct 31 2011 Rex Dieter <rdieter@fedoraproject.org> 4.6.2-1.fc17.1
- rebuild (gmp)
